@@ -1,35 +1,26 @@
-/********************************************************************
-Copyright 2016  Eike Hein <hein.org>
+/*
+    SPDX-FileCopyrightText: 2016 Eike Hein <hein@kde.org>
 
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) version 3, or any
-later version accepted by the membership of KDE e.V. (or its
-successor approved by the membership of KDE e.V.), which shall
-act as a proxy defined in Section 6 of version 3 of the license.
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
-
-#ifndef TASKSMODEL_H
-#define TASKSMODEL_H
+#pragma once
 
 #include <QQmlParserStatus>
 #include <QSortFilterProxyModel>
 
+#include <memory>
+
 #include "abstracttasksmodeliface.h"
+#include "regionfiltermode.h"
 
 #include "taskmanager_export.h"
 
 namespace TaskManager
 {
+class ActivityInfo;
+class VirtualDesktopInfo;
+
 /**
  * @short A unified tasks model.
  *
@@ -56,6 +47,7 @@ namespace TaskManager
 class TASKMANAGER_EXPORT TasksModel : public QSortFilterProxyModel, public AbstractTasksModelIface, public QQmlParserStatus
 {
     Q_OBJECT
+    QML_ELEMENT
     Q_INTERFACES(QQmlParserStatus)
 
     Q_PROPERTY(int count READ rowCount NOTIFY countChanged)
@@ -67,16 +59,22 @@ class TASKMANAGER_EXPORT TasksModel : public QSortFilterProxyModel, public Abstr
 
     Q_PROPERTY(QVariant virtualDesktop READ virtualDesktop WRITE setVirtualDesktop NOTIFY virtualDesktopChanged)
     Q_PROPERTY(QRect screenGeometry READ screenGeometry WRITE setScreenGeometry NOTIFY screenGeometryChanged)
+    Q_PROPERTY(QRect regionGeometry READ regionGeometry WRITE setRegionGeometry NOTIFY regionGeometryChanged)
     Q_PROPERTY(QString activity READ activity WRITE setActivity NOTIFY activityChanged)
 
     Q_PROPERTY(bool filterByVirtualDesktop READ filterByVirtualDesktop WRITE setFilterByVirtualDesktop NOTIFY filterByVirtualDesktopChanged)
     Q_PROPERTY(bool filterByScreen READ filterByScreen WRITE setFilterByScreen NOTIFY filterByScreenChanged)
     Q_PROPERTY(bool filterByActivity READ filterByActivity WRITE setFilterByActivity NOTIFY filterByActivityChanged)
+    Q_PROPERTY(RegionFilterMode::Mode filterByRegion READ filterByRegion WRITE setFilterByRegion NOTIFY filterByRegionChanged)
+    Q_PROPERTY(bool filterMinimized READ filterMinimized WRITE setFilterMinimized NOTIFY filterMinimizedChanged)
     Q_PROPERTY(bool filterNotMinimized READ filterNotMinimized WRITE setFilterNotMinimized NOTIFY filterNotMinimizedChanged)
+    Q_PROPERTY(bool filterNotMaximized READ filterNotMaximized WRITE setFilterNotMaximized NOTIFY filterNotMaximizedChanged)
+    Q_PROPERTY(bool filterHidden READ filterHidden WRITE setFilterHidden NOTIFY filterHiddenChanged)
 
     Q_PROPERTY(SortMode sortMode READ sortMode WRITE setSortMode NOTIFY sortModeChanged)
     Q_PROPERTY(bool separateLaunchers READ separateLaunchers WRITE setSeparateLaunchers NOTIFY separateLaunchersChanged)
     Q_PROPERTY(bool launchInPlace READ launchInPlace WRITE setLaunchInPlace NOTIFY launchInPlaceChanged)
+    Q_PROPERTY(bool hideActivatedLaunchers READ hideActivatedLaunchers WRITE setHideActivatedLaunchers NOTIFY hideActivatedLaunchersChanged)
 
     Q_PROPERTY(GroupMode groupMode READ groupMode WRITE setGroupMode NOTIFY groupModeChanged)
     Q_PROPERTY(bool groupInline READ groupInline WRITE setGroupInline NOTIFY groupInlineChanged)
@@ -85,6 +83,7 @@ class TASKMANAGER_EXPORT TasksModel : public QSortFilterProxyModel, public Abstr
     Q_PROPERTY(QStringList groupingAppIdBlacklist READ groupingAppIdBlacklist WRITE setGroupingAppIdBlacklist NOTIFY groupingAppIdBlacklistChanged)
     Q_PROPERTY(QStringList groupingLauncherUrlBlacklist READ groupingLauncherUrlBlacklist WRITE setGroupingLauncherUrlBlacklist NOTIFY
                    groupingLauncherUrlBlacklistChanged)
+    Q_PROPERTY(bool taskReorderingEnabled READ taskReorderingEnabled WRITE setTaskReorderingEnabled NOTIFY taskReorderingEnabledChanged)
     Q_PROPERTY(QModelIndex activeTask READ activeTask NOTIFY activeTaskChanged)
 
 public:
@@ -94,6 +93,8 @@ public:
         SortAlpha, /**< Tasks are sorted alphabetically, by AbstractTasksModel::AppName and Qt::DisplayRole. */
         SortVirtualDesktop, /**< Tasks are sorted by the virtual desktop they are on. */
         SortActivity, /**< Tasks are sorted by the number of tasks on the activities they're on. */
+        SortLastActivated, /**< Tasks are sorted by the last time they were active. */
+        SortWindowPositionHorizontal, /**< Tasks are sorted by the virtual desktop they are on, then by window coordinates. */
     };
     Q_ENUM(SortMode)
 
@@ -108,12 +109,12 @@ public:
 
     QHash<int, QByteArray> roleNames() const override;
 
-    Q_INVOKABLE int rowCount(const QModelIndex &parent = QModelIndex()) const override; // Invokable.
+    Q_INVOKABLE int rowCount(const QModelIndex &parent = QModelIndex()) const override; // Invocable.
 
     QVariant data(const QModelIndex &proxyIndex, int role) const override;
 
     /**
-     * The number of launcher tasks in the tast list.
+     * The number of launcher tasks in the task list.
      *
      * @returns the number of launcher tasks in the task list.
      **/
@@ -187,6 +188,27 @@ public:
      **/
     void setScreenGeometry(const QRect &geometry);
 
+    /**
+     * The geometry of the region used in filtering by region. Defaults
+     * to a null QRect.
+     *
+     * @see setRegionGeometry
+     * @since 6.0
+     * @returns the geometry of the region used in filtering.
+     **/
+    QRect regionGeometry() const;
+
+    /**
+     * Set the geometry of the screen to use in filtering by region.
+     *
+     * If set to an invalid QRect, filtering by region is disabled.
+     *
+     * @see regionGeometry
+     * @since 6.0
+     * @param geometry A region geometry.
+     **/
+    void setRegionGeometry(const QRect &geometry);
+
     /**
      * The id of the activity used in filtering by activity. Usually
      * set to the id of the current activity. Defaults to an empty id.
@@ -236,7 +258,7 @@ public:
      * if this returns @c true.
      *
      * @see setFilterByScreen
-     * @see setScreen
+     * @see setScreenGeometry
      * @returns @c true if tasks should be filtered by screen.
      **/
     bool filterByScreen() const;
@@ -248,7 +270,7 @@ public:
      * if this is set to @c true.
      *
      * @see filterByScreen
-     * @see setScreen
+     * @see setScreenGeometry
      * @param filter Whether tasks should be filtered by screen.
      **/
     void setFilterByScreen(bool filter);
@@ -261,7 +283,7 @@ public:
      *
      * @see setFilterByActivity
      * @see setActivity
-     * @returns @ctrue if tasks should be filtered by activity.
+     * @returns @c true if tasks should be filtered by activity.
      **/
     bool filterByActivity() const;
 
@@ -269,7 +291,7 @@ public:
      * Set whether tasks should be filtered by activity. Defaults to
      * @c false.
      *
-     * Filtering by virtual desktop only happens if an activity id is set,
+     * Filtering by activity only happens if an activity id is set,
      * even if this is set to @c true.
      *
      * @see filterByActivity
@@ -278,6 +300,54 @@ public:
      **/
     void setFilterByActivity(bool filter);
 
+    /**
+     * Whether tasks should be filtered by region. Defaults to @c RegionFilterMode::Disabled.
+     *
+     * Filtering by region only happens if a region is set, even
+     * if the filter is enabled.
+     *
+     * @see RegionFilterMode
+     * @see setFilterByRegion
+     * @see setRegionGeometry
+     * @since 6.0
+     * @returns Region filter mode.
+     **/
+    RegionFilterMode::Mode filterByRegion() const;
+
+    /**
+     * Set whether tasks should be filtered by region. Defaults to
+     * @c RegionFilterMode::Disabled.
+     *
+     * Filtering by region only happens if a region is set,
+     * even if the filter is enabled.
+     *
+     * @see RegionFilterMode
+     * @see filterByRegion
+     * @see setRegionGeometry
+     * @since 6.0
+     * @param mode Region filter mode.
+     **/
+    void setFilterByRegion(RegionFilterMode::Mode mode);
+
+    /**
+     * Whether minimized tasks should be filtered out. Defaults to
+     * @c false.
+     *
+     * @returns @c true if minimized tasks should be filtered out.
+     * @see setFilterMinimized
+     * @since 5.27
+     **/
+    bool filterMinimized() const;
+
+    /**
+     * Sets whether non-minimized tasks should be filtered out.
+     *
+     * @param filter Whether minimized tasks should be filtered out.
+     * @see filterMinimized
+     * @since 5.27
+     **/
+    void setFilterMinimized(bool filter);
+
     /**
      * Whether non-minimized tasks should be filtered. Defaults to
      * @c false.
@@ -295,6 +365,40 @@ public:
      **/
     void setFilterNotMinimized(bool filter);
 
+    /**
+     * Whether non-maximized tasks should be filtered. Defaults to
+     * @c false.
+     *
+     * @see setFilterNotMaximized
+     * @returns @c true if non-maximized tasks should be filtered.
+     **/
+    bool filterNotMaximized() const;
+
+    /**
+     * Set whether non-maximized tasks should be filtered.
+     *
+     * @see filterNotMaximized
+     * @param filter Whether non-maximized tasks should be filtered.
+     **/
+    void setFilterNotMaximized(bool filter);
+
+    /**
+     * Whether hidden tasks should be filtered. Defaults to
+     * @c false.
+     *
+     * @see setFilterHidden
+     * @returns @c true if hidden tasks should be filtered.
+     **/
+    bool filterHidden() const;
+
+    /**
+     * Set whether hidden tasks should be filtered.
+     *
+     * @see filterHidden
+     * @param filter Whether hidden tasks should be filtered.
+     **/
+    void setFilterHidden(bool filter);
+
     /**
      * The sort mode used in sorting tasks. Defaults to SortAlpha.
      *
@@ -376,6 +480,26 @@ public:
      **/
     void setLaunchInPlace(bool launchInPlace);
 
+    /**
+     * Whether launchers should be hidden after they have been
+     * activated. Defaults to @c true.
+     *
+     * @see hideActivatedLaunchers
+     * @returns whether launchers will be hidden after they have been
+     * activated.
+     **/
+    bool hideActivatedLaunchers() const;
+
+    /**
+     * Sets whether launchers should be hidden after they have been
+     * activated. Defaults to @c true.
+     *
+     * @see hideActivatedLaunchers
+     * @param hideActivatedLaunchers Whether launchers should be hidden
+     * after they have been activated.
+     **/
+    void setHideActivatedLaunchers(bool hideActivatedLaunchers);
+
     /**
      * Returns the current group mode, i.e. the criteria by which tasks should
      * be grouped.
@@ -512,6 +636,20 @@ public:
      **/
     void setGroupingLauncherUrlBlacklist(const QStringList &list);
 
+    /**
+     * Enables or disables tasks reordering.
+     *
+     * @param enabled enables tasks reordering if @c true; disables it otherwise.
+     */
+    void setTaskReorderingEnabled(bool enabled);
+
+    /**
+     * Returns whether tasks reordering is enabled or not.
+     *
+     * @returns whether tasks reordering is enabled or not.
+     */
+    bool taskReorderingEnabled() const;
+
     /**
      * Finds the first active (AbstractTasksModel::IsActive) task in the model
      * and returns its QModelIndex, or a null QModelIndex if no active task is
@@ -709,6 +847,28 @@ public:
      **/
     Q_INVOKABLE void requestToggleShaded(const QModelIndex &index) override;
 
+    /**
+     * Request toggling the no border state of the task at given index.
+     *
+     * This is meant for tasks that have an associated window, and may be
+     * a no-op when there is no window.
+     *
+     * @param index An index in this tasks model.
+     * @since 6.4
+     */
+    Q_INVOKABLE void requestToggleNoBorder(const QModelIndex &index) override;
+
+    /**
+     * Request toggling the excludeFromCapture state of the task at given index.
+     *
+     * This is meant for tasks that have an associated window, and may be
+     * a no-op when there is no window.
+     *
+     * @param index An index in this tasks model.
+     * @since 6.6
+     **/
+    Q_INVOKABLE void requestToggleExcludeFromCapture(const QModelIndex &index) override;
+
     /**
      * Request entering the window at the given index on the specified virtual desktops.
      *
@@ -843,34 +1003,49 @@ Q_SIGNALS:
     void anyTaskDemandsAttentionChanged() const;
     void virtualDesktopChanged() const;
     void screenGeometryChanged() const;
+    void regionGeometryChanged();
     void activityChanged() const;
     void filterByVirtualDesktopChanged() const;
     void filterByScreenChanged() const;
     void filterByActivityChanged() const;
+    void filterByRegionChanged();
+    void filterMinimizedChanged();
     void filterNotMinimizedChanged() const;
+    void filterNotMaximizedChanged() const;
+    void filterHiddenChanged() const;
     void sortModeChanged() const;
     void separateLaunchersChanged() const;
     void launchInPlaceChanged() const;
+    void hideActivatedLaunchersChanged() const;
     void groupModeChanged() const;
     void groupInlineChanged() const;
     void groupingWindowTasksThresholdChanged() const;
     void groupingAppIdBlacklistChanged() const;
     void groupingLauncherUrlBlacklistChanged() const;
+    void taskReorderingEnabledChanged() const;
     void activeTaskChanged() const;
 
 protected:
     bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const override;
     bool lessThan(const QModelIndex &left, const QModelIndex &right) const override;
 
+    /**
+     * @return a shared pointer to VirtualDesktopInfo to access virtual desktop information
+     */
+    std::shared_ptr<VirtualDesktopInfo> virtualDesktopInfo() const;
+
+    /**
+     * @return a shared pointer to ActivityInfo to access activity information
+     */
+    std::shared_ptr<ActivityInfo> activityInfo() const;
+
 private:
     Q_INVOKABLE void updateLauncherCount();
 
     class Private;
     class TasksModelLessThan;
     friend class TasksModelLessThan;
-    QScopedPointer<Private> d;
+    std::unique_ptr<Private> d;
 };
 
 }
-
-#endif
